@@ -133,6 +133,7 @@ class Tube:
         self._socket: Socket = None
         self.context = Context().instance()
         self.tube_info = kwargs
+        self.is_closed = False
         self._sockopts = {}
         self.sockopts = kwargs.get('sockopts', {})
         self.addr = kwargs.get('addr')
@@ -224,6 +225,10 @@ class Tube:
         return self.tube_type in [zmq.PUB, zmq.SUB, zmq.REP, zmq.ROUTER,
                                   zmq.DEALER]
 
+    @property
+    def is_connected(self):
+        return self._socket is not None
+
     @property
     def raw_socket(self) -> Socket:
         """
@@ -271,7 +276,7 @@ class Tube:
         if self.is_server:
             self.logger.debug(
                 f"The tube '{self.name}' (ZMQ.{self.tube_type_name}) "
-                f"binds to the port {self.addr}.")
+                f"binds to the port {self.addr}")
             raw_socket.bind(self.addr)
         else:
             self.logger.debug(
@@ -288,7 +293,9 @@ class Tube:
         For persistent tubes, this open connection (connect/bind) to address.
         """
         if self.is_persistent and self._socket is None:
-            return self.raw_socket
+            self.raw_socket
+        self.is_closed = False
+        return self
 
     def close(self):
         """
@@ -296,6 +303,7 @@ class Tube:
         """
         if self.is_persistent and self._socket:
             self.raw_socket.close()
+        self.is_closed = True
 
     @singledispatchmethod
     def send(self, arg):
@@ -382,6 +390,8 @@ class Tube:
 
 class TubeNode:
 
+    __TUBE_CLASS = Tube
+
     def __init__(self, *, schema=None, warning_not_mach_topic=True):
         self.logger = logging.getLogger(self.__class__.__name__)
         self._tubes = TopicMatcher()
@@ -391,6 +401,14 @@ class TubeNode:
         self._stop_main_loop = False
         self.warning_not_mach_topic = warning_not_mach_topic
 
+    def __enter__(self):
+        self.connect()
+        self.start()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.stop()
+        self.close()
+
     @property
     def tubes(self) -> [Tube]:
         """
@@ -418,7 +436,7 @@ class TubeNode:
         """
         if 'tubes' in schema:
             for tube_info in schema['tubes']:
-                tube = Tube(**tube_info)
+                tube = self.__TUBE_CLASS(**tube_info)
                 self.register_tube(tube, tube_info.get('topics', []))
 
     def get_tube_by_topic(self, topic: str, types=None) -> Tube:
@@ -427,7 +445,7 @@ class TubeNode:
         Optional: we can specify a type of tube.
         """
         res = self._tubes.match(topic)
-        if types:
+        if res and types:
             res = [t for t in res if t.tube_type in types]
         if not res:
             return None
@@ -435,6 +453,15 @@ class TubeNode:
             res = res.pop()
         return res
 
+    def filter_tube_by_topic(self, topic: str, types=None) -> [(str, Tube)]:
+        tubes = self._tubes.filter(topic)
+        res = {}
+        for top, tts in tubes:
+            for tt in tts:
+                if not types or tt.tube_type in types:
+                    res[top] = tt
+        return res
+
     def get_tube_by_name(self, name: str) -> Tube:
         """
         returns the Tube with the name
@@ -452,20 +479,26 @@ class TubeNode:
         if isinstance(topics, str):
             topics = [topics]
         for topic in topics:
-            tubes = self._tubes.get_topic(topic)
-            if tubes:
-                self.logger.info(f"The tube '{tube.name}' overrides "
-                                 f"the exist topic: {topic}")
-                tubes.append(tube)
-            else:
-                self.logger.debug(f"The tube '{tube.name}' was registered to "
-                                  f"the topic: {topic}")
-                tubes = [tube, ]
+            tubes = self._tubes.get_topic(topic) or []
+            tubes.append(tube)
+            self.logger.debug(f"The tube '{tube.name}' was registered to "
+                              f"the topic: {topic}")
             self._tubes.set_topic(topic, tubes)
 
-    def get_callback_by_topic(self, topic: str) -> Callable:
-        res = self._callbacks.match(topic)
-        return res
+    def get_callback_by_topic(self, topic: str, tube=None) -> Callable:
+        """
+        This return callbacks for the topic.
+        If any of the callbacks is assigned to the tube,
+        it is returned only these. Otherwise, this returns all unassigned.
+        """
+        callbacks = []
+        callbacks_for_tube = []
+        for clb in self._callbacks.match(topic) or []:
+            if 'tube' not in clb.__dict__:
+                callbacks.append(clb)
+            elif clb.__dict__['tube'] == tube:
+                callbacks_for_tube.append(clb)
+        return callbacks_for_tube if callbacks_for_tube else callbacks
 
     def send(self, topic: str, payload=None, tube=None):
         if not tube:
@@ -486,6 +519,10 @@ class TubeNode:
         return res
 
     def publish(self, topic: str, payload=None):
+        """
+        In the case with asyncio, the first message is very often lost.
+        The workaround is to connect the tube manually as soon as possible.
+        """
         tube = self.get_tube_by_topic(topic, [zmq.PUB])
         if not tube:
             raise TubeTopicNotConfigured(f'The topic "{topic}" is not assigned '
@@ -493,20 +530,26 @@ class TubeNode:
         self.send(topic, payload, tube)
 
     def subscribe(self, topic: str, fce: Callable):
-        tube = self.get_tube_by_topic(topic, [zmq.SUB])
-        if not tube:
+        topic_tubes = self.filter_tube_by_topic(topic, [zmq.SUB])
+        if not topic_tubes:
             raise TubeTopicNotConfigured(f'The topic "{topic}" is not assigned '
                                          f'to any Tube for subscribe.')
-        self.register_handler(topic, fce, tube=tube)
+        for tube_topic, tube in topic_tubes.items():
+            self.register_handler(tube_topic, fce, tube=tube)
 
     def register_handler(self, topic: str, fce: Callable, tube: Tube = None):
-        if not tube:
-            tube = self.get_tube_by_topic(topic, [zmq.SUB, zmq.REP,
-                                                  zmq.ROUTER, zmq.DEALER])
-        if not tube:
-            self.logger.warning(f"This topic '{topic}' does not have any "
-                                f"assigned tube. This callback will be "
-                                f"never fired.")
+        """
+        We can register more handlers for SUB and all will be executed.
+        For REP, ROUTER and DEALER, there will be executed only
+        the last registered.
+        If we want to use DEALER as server and client on the same node,
+        we have to specify which tube will be used for this handler.
+        :param topic: str
+        :param fce: Callable
+        :param tube: Tube - only for the case DEALER x DEALER on the same node.
+        """
+        if tube:
+            fce.__dict__['tube'] = tube
         self._callbacks.get_topic(topic, set_default=[]).append(fce)
 
     def stop(self):
@@ -519,9 +562,9 @@ class TubeNode:
                 if _request.tube.tube_type in [zmq.ROUTER]:
                     raise TubeMessageError(
                         f"The response of the {_request.tube.tube_type_name} "
-                        f"callback has to be a instamce of TubeMessage class.")
+                        f"callback has to be a instance of TubeMessage class.")
                 _payload = response
-                response = request.create_response()
+                response = _request.create_response()
                 response.payload = _payload
             else:
                 if _request.tube.tube_type in [zmq.ROUTER] and\
@@ -529,7 +572,7 @@ class TubeNode:
                     raise TubeMessageError(
                         "The TubeMessage response object doesn't be created "
                         "from request object.")
-            request.tube.send(response)
+            _request.tube.send(response)
 
         poller = Poller()
         loop = asyncio.get_event_loop()
@@ -549,7 +592,7 @@ class TubeNode:
                 raw_socket = event[0]
                 tube: Tube = raw_socket.__dict__['tube']
                 request = await tube.receive_data(raw_socket=raw_socket)
-                callbacks = self.get_callback_by_topic(request.topic)
+                callbacks = self.get_callback_by_topic(request.topic, tube)
                 if not callbacks:
                     if self.warning_not_mach_topic:
                         self.logger.warning(
@@ -557,8 +600,6 @@ class TubeNode:
                             f"it is ignored (topic: {request.topic})"
                         )
                     continue
-                # self.logger.debug(
-                #     f"Incoming message for tube '{tube.name}'")
                 if tube.tube_type == zmq.SUB:
                     for callback in callbacks:
                         loop.create_task(callback(request), name='zmq/sub')
